import java.io.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.bluetooth.*;


/**
 * A very simple GPS simulator.
 *
 * @author Michel Deriaz
 */
public class GPSSim extends MIDlet implements Runnable, CommandListener {
  public static final UUID SERVICE_ID = new UUID("BAE0D0C0B0A000955570605040302010", false);
  public static final int INTERVAL = 500; // refresh delay
  
  Form form;
  TextField tfUTC, tfLat, tfLon, tfAlt, tfNS, tfEW, tfQuality, tfNumSat, tfHorDil, tfMetersHeight, tfGeoSep, tfMetersGeo, tfAge, tfDiffStationID, tfChecksum;
  TextField tfVTG;
  StreamConnectionNotifier scn;


  public GPSSim() {
    form = new Form("GPS Simulator");
    tfUTC = new TextField("UTC:", "083549.003", 10, TextField.DECIMAL);
    tfUTC.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfUTC);
    tfLat = new TextField("Lat:", "4611.9761", 10, TextField.ANY);
    tfLat.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfLat);
    tfLon = new TextField("Lon:", "00608.5362", 10, TextField.ANY);
    tfLon.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfLon);
    tfAlt = new TextField("Alt:", "412.8", 10, TextField.DECIMAL);
    tfAlt.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfAlt);
    tfNS = new TextField("N or S:", "N", 10, TextField.ANY | TextField.INITIAL_CAPS_SENTENCE);
    tfNS.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfNS);
    tfEW = new TextField("E or W:", "E", 10, TextField.ANY | TextField.INITIAL_CAPS_SENTENCE);
    tfEW.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfEW);
    tfQuality = new TextField("Quality (0, 1 or 2):", "1", 10, TextField.DECIMAL);
    tfQuality.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfQuality);
    tfNumSat = new TextField("Number of sat. in use:", "6", 10, TextField.DECIMAL);
    tfNumSat.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfNumSat);
    tfHorDil = new TextField("Horizontal dilution:", "3.0", 10, TextField.DECIMAL);
    tfHorDil.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfHorDil);
    tfMetersHeight = new TextField("Meters (height):", "M", 10, TextField.ANY | TextField.INITIAL_CAPS_SENTENCE);
    tfMetersHeight.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfMetersHeight);
    tfGeoSep = new TextField("Geo. sep.:", "48.2", 10, TextField.DECIMAL);
    tfGeoSep.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfGeoSep);
    tfMetersGeo = new TextField("Meters (geo):", "M", 10, TextField.ANY | TextField.INITIAL_CAPS_SENTENCE);
    tfMetersGeo.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfMetersGeo);
    tfAge = new TextField("Age (since last diff.):", "", 10, TextField.DECIMAL);
    tfAge.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfAge);
    tfDiffStationID = new TextField("Diff. station ID:", "0000*53", 10, TextField.ANY);
    tfDiffStationID.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfDiffStationID);
    tfChecksum = new TextField("Checksum:", "", 10, TextField.ANY);
    tfChecksum.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfChecksum);
    form.append(new Spacer(0, 25));
    try {
      form.append(new ImageItem(null, Image.createImage("/separator.png"), ImageItem.LAYOUT_2 | ImageItem.LAYOUT_CENTER, null));
    }
    catch (IOException e) {}
    form.append(new Spacer(0, 25));
    tfVTG = new TextField("VTG:", "$GPVTG,054.7,T,034.4,M,005.5,N,010.2,K", 200, TextField.ANY);
    tfVTG.setLayout(Item.LAYOUT_2 | Item.LAYOUT_NEWLINE_AFTER);
    form.append(tfVTG);
    form.addCommand(new Command("Exit", Command.EXIT, 0));
    form.setCommandListener(this);
    Thread runner = new Thread(this);
    runner.start();
  }

  
  public void commandAction(Command c, Displayable s) {
    if (c.getCommandType() == Command.EXIT) {
      try {
        scn.close();
      }
      catch (Exception e) {
        System.out.println("Exception in commandAction: " + e);
      }
      notifyDestroyed();
    }
  }

 
  public void run() {
    scn = null;
    try {
      LocalDevice localDevice = LocalDevice.getLocalDevice();
      localDevice.setDiscoverable(DiscoveryAgent.GIAC);
      String url = "btspp://localhost:" + SERVICE_ID.toString() + ";name=GPS Simulator;authorize=false";
      scn = (StreamConnectionNotifier)Connector.open(url);
    }
    catch (Exception e) {
      System.out.println("Exception during initilization: " + e);
      return;
    }
    StreamConnection streamCon = null;
    while (true) {
      try {
        streamCon = scn.acceptAndOpen();
      }
      catch (IOException e) { // thrown when scn.close() is invoked
        return;
      }
      DataOutputStream dos = null;
      try {
        dos = streamCon.openDataOutputStream();
        while (true) {
          dos.writeUTF(buildLine());
          dos.flush();
          Thread.sleep(INTERVAL);
          dos.writeUTF(tfVTG.getString());
          dos.flush();
          Thread.sleep(INTERVAL);
        }
      }
      catch (Exception e) { // thrown if client disappears
        System.out.println("End of connection");
        try {
          dos.close();
          streamCon.close();
        }
        catch (Exception e2) {}
      }
    }
  }
  
  
  protected void startApp() {
    Display.getDisplay(this).setCurrent(form);
  }
  

  protected void pauseApp () {}
  protected void destroyApp(boolean unconditional) {}

  
  private String buildLine() {
    StringBuffer sb = new StringBuffer("$GPGGA");
    sb.append("," + tfUTC.getString());
    sb.append("," + tfLat.getString());
    sb.append("," + tfNS.getString());
    sb.append("," + tfLon.getString());
    sb.append("," + tfEW.getString());
    sb.append("," + tfQuality.getString());
    sb.append("," + tfNumSat.getString());
    sb.append("," + tfHorDil.getString());
    sb.append("," + tfAlt.getString());
    sb.append("," + tfMetersHeight.getString());
    sb.append("," + tfGeoSep.getString());
    sb.append("," + tfMetersGeo.getString());
    sb.append("," + tfAge.getString());
    sb.append("," + tfDiffStationID.getString());
    if (!tfChecksum.getString().equals("")) sb.append("," + tfChecksum.getString());
    return sb.toString();  
  }
}